@@ -1,12 +1,23 @@
 use std::ffi::{self, CStr, CString, OsStr, OsString};
 use std::os::unix::ffi::OsStrExt;
-use std::str::FromStr;
+use std::str::{self, FromStr};
 
 use super::{NulError, PdCStr, PdCString};
 
 pub(crate) type PdCStringInner = CString;
 pub(crate) type PdCStrInner = CStr;
 
+#[doc(hidden)]
+pub extern crate cstr;
+
+#[macro_export]
+/// A macro for creating a [`PdCStr`](crate::pdcstring::PdCStr) at compile time.
+macro_rules! pdcstr {
+    ($expression:expr) => {
+        $crate::pdcstring::PdCStr::from_c_str($crate::pdcstring::cstr::cstr!($expression))
+    };
+}
+
 // conversions to and from inner
 impl PdCString {
     pub fn from_c_string(s: CString) -> Self {
@@ -23,7 +34,7 @@ impl PdCString {
         PdCString::from_vec(s.as_ref().as_bytes().to_vec())
     }
     pub unsafe fn from_str_ptr(ptr: *const i8) -> Self {
-        PdCStr::from_str_ptr(ptr).to_owned()
+        unsafe { PdCStr::from_str_ptr(ptr) }.to_owned()
     }
 }
 
@@ -65,11 +76,11 @@ impl PdCStr {
         self.0.as_ptr()
     }
     pub unsafe fn from_str_ptr<'a>(ptr: *const i8) -> &'a Self {
-        let inner = CStr::from_ptr(ptr);
+        let inner = unsafe { CStr::from_ptr(ptr) };
         PdCStr::from_inner(inner)
     }
     pub unsafe fn from_slice_with_nul_unchecked(slice: &[u8]) -> &Self {
-        let inner = CStr::from_bytes_with_nul_unchecked(slice);
+        let inner = unsafe { CStr::from_bytes_with_nul_unchecked(slice) };
         PdCStr::from_inner(inner)
     }
     pub fn to_os_string(&self) -> OsString {
@@ -92,4 +103,13 @@ impl PdCStr {
     pub fn to_slice_with_nul(&self) -> &[u8] {
         self.0.to_bytes_with_nul()
     }
+    pub fn len(&self) -> usize {
+        self.0.to_bytes().len()
+    }
+    pub fn to_string(&self) -> Result<String, str::Utf8Error> {
+        self.0.to_str().map(|s| s.to_string())
+    }
+    pub fn to_string_lossy(&self) -> String {
+        self.0.to_string_lossy().to_string()
+    }
 }
